@@ -72,7 +72,7 @@ namespace FSpot {
 		public void Adjust ()
 		{
 			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
-			ImageFile img = ImageFile.Create (photo.DefaultVersionPath);
+			ImageFile img = ImageFile.Create (photo.DefaultVersionUri);
 
 			if (image == null)
 				image = img.Load ();
@@ -108,11 +108,12 @@ namespace FSpot {
 		}
 	}
 
-
 	public class ColorDialog : GladeDialog {
+		protected static ColorDialog instance = null;
+		
 		Gdk.Pixbuf ScaledPixbuf;
 		Gdk.Pixbuf AdjustedPixbuf;
-
+		
 #if USE_THREAD		
 		Delay expose_timeout;
 #endif
@@ -134,10 +135,10 @@ namespace FSpot {
 		
 		[Glade.Widget] private Gtk.ScrolledWindow view_scrolled;
 		[Glade.Widget] private Gtk.Image histogram_image;
-
+		
 		[Glade.Widget] private Gtk.CheckButton white_check;
 		[Glade.Widget] private Gtk.CheckButton exposure_check;
-
+		
 		[Glade.Widget] Gtk.Button ok_button;
 		[Glade.Widget] Gtk.VBox   control_vbox;
 		
@@ -203,6 +204,10 @@ namespace FSpot {
 			}
 		}
 
+		public static ColorDialog Instance {
+			get { return instance; }
+		}
+		
 		public bool UseWhiteSettings {
 			get {
 				if (white_check != null)
@@ -328,6 +333,7 @@ namespace FSpot {
 			view.QueueDraw ();
 			System.Console.WriteLine ("clearing window");
 			this.Dialog.Destroy ();
+            instance = null;
 		}
 		
 		private void HandleOkClicked (object sender, EventArgs args)
@@ -349,16 +355,16 @@ namespace FSpot {
 		{
 			try {
 				if (!view.Item.IsValid)
-					throw new Exception ("Invalid Image");
+ 					throw new Exception ("Invalid Image");
+				
+				FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionUri);
 			
-				FSpot.ImageFile img = FSpot.ImageFile.Create (((Photo)view.Item.Current).DefaultVersionPath);
-
-				try {
-					image_profile = img.GetProfile ();
-				} catch (System.Exception e) {
-					image_profile = null;
-					System.Console.WriteLine (e);
-				}
+ 				try {
+ 					image_profile = img.GetProfile ();
+ 				} catch (System.Exception e) {
+ 					image_profile = null;
+ 					System.Console.WriteLine (e);
+ 				}
 
 				// FIXME fall back to rgb for now
 				if (image_profile == null)
@@ -367,21 +373,20 @@ namespace FSpot {
 				AdjustedPixbuf = img.Load (256, 256);
 				ScaledPixbuf = AdjustedPixbuf.Copy ();			
 
-				// FIXME handle alpha
 				if (AdjustedPixbuf.HasAlpha)
 					throw new Exception ("Unsupported Alpha Channel");
 
-				control_vbox.Sensitive = true;
-				ok_button.Sensitive = true;
+ 				control_vbox.Sensitive = true;
+ 				ok_button.Sensitive = true;
 
-				RangeChanged (null, null);
-			} catch (System.Exception e) {
-				control_vbox.Sensitive = false;
-				ok_button.Sensitive = false;
-				AdjustedPixbuf = null;
-				ScaledPixbuf = null;
-				image_profile = null;
-			}
+  				RangeChanged (null, null);
+  			} catch (System.Exception e) {
+ 				control_vbox.Sensitive = false;
+ 				ok_button.Sensitive = false;
+ 				AdjustedPixbuf = null;
+ 				ScaledPixbuf = null;
+  				image_profile = null;
+  			}	
 		}
 
 		private const double e = 0.0;
@@ -492,24 +497,59 @@ namespace FSpot {
 			Cancel ();
 		}
 		
-		public ColorDialog (FSpot.PhotoQuery query, int item)
+		public static void Close ()
+		{
+			if (instance != null) {
+				instance.Cancel ();
+			}
+		}
+		
+		public static void CreateForView (FSpot.PhotoImageView view)
+		{
+			Close ();
+			
+			instance = new ColorDialog (view);
+		}
+		
+		// FIXME is this the riht place for this, shouldn't mainwindow
+		// treat fullscreen as a mode and handle this part itself?
+		public static void SwitchViews (FSpot.PhotoImageView view) {
+			if (instance != null) {
+				if (instance.view.Item.Current == view.Item.Current) {
+					instance.view.Transform = null;
+					instance.SetView (view);
+					instance.Adjust ();
+				} else {
+					CreateForView (view);
+				}
+			}
+		}
+		
+		protected ColorDialog (FSpot.PhotoQuery query, int item)
 		{
 			view = new FSpot.PhotoImageView (query);
 			view_scrolled.Add (view);
 			view.Show ();
 			view.Item.Index = item;
-
+			
 			this.CreateDialog ("external_color_dialog");
 			AttachInterface ();
 		}
-
-		public ColorDialog (FSpot.PhotoImageView view)       
+		
+		protected ColorDialog (FSpot.PhotoImageView view)       
 		{
 			this.view = view;
-			this.CreateDialog ("inline_color_dialog");
-			AttachInterface ();
-		}
 
+			Window parent = view.Toplevel as Window;
+
+			this.CreateDialog ("inline_color_dialog");
+
+			AttachInterface ();
+			if (parent != null)
+				this.Dialog.TransientFor = parent;
+
+		}
+		
 		private void AttachInterface ()
 		{
 			view.PhotoChanged += HandlePhotoChanged;
@@ -556,5 +596,13 @@ namespace FSpot {
 
 			HandlePhotoChanged (view);
 		}
+
+		private void SetView (FSpot.PhotoImageView view)
+        {
+			this.view.PhotoChanged -= HandlePhotoChanged;
+            this.view = view;
+			this.view.PhotoChanged += HandlePhotoChanged;
+        }
+
 	}
 }
