@@ -7,6 +7,8 @@ using System.Collections.Specialized;
 using System.Web;
 using Mono.Unix;
 
+using FSpot.Filters;
+
 using GalleryRemote;
 
 namespace FSpot {
@@ -45,11 +47,10 @@ namespace FSpot {
 
 			System.Console.WriteLine ("Gallery created: " + gal);
 
-			if (gal.Login (username, password)) {
-				//System.Console.WriteLine ("Login successful");
-				gallery = gal;
-				connected = true;
-			}
+			gal.Login (username, password);
+
+			gallery = gal;
+			connected = true;
 
 			return gallery;
 		}
@@ -356,8 +357,17 @@ namespace FSpot {
 					if (uri.Scheme != Uri.UriSchemeHttp &&
 					    uri.Scheme != Uri.UriSchemeHttps)
 						throw new System.UriFormatException ();
-
-				} catch (System.UriFormatException e) {
+					
+					GalleryAccount created = new GalleryAccount (name, 
+										     url, 
+										     username,
+										     password);
+					
+					created.Connect ();
+					GalleryAccountManager.GetInstance ().AddAccount (created);
+					account = created;
+				} catch (System.UriFormatException) {
+					
 					HigMessageDialog md = 
 						new HigMessageDialog (Dialog, 
 								      Gtk.DialogFlags.Modal |
@@ -368,13 +378,29 @@ namespace FSpot {
 					md.Run ();
 					md.Destroy ();
 					return;
+				} catch (GalleryRemote.GalleryCommandException e) {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Error while connecting to Gallery"),
+								      String.Format (Catalog.GetString ("The following error was encountered while attempting to log in: {0}"), e.Message));
+					md.Run ();
+					md.Destroy ();
+					return;
+				} catch (System.Net.WebException we) {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Error while connecting to Gallery"),
+								      String.Format (Catalog.GetString ("The following error was encountered while attempting to log in: {0}"), we.Message));
+					md.Run ();
+					md.Destroy ();
+					return;
 				}
-
-				GalleryAccount account = new GalleryAccount (name, 
-									     url, 
-									     username,
-									     password);
-				GalleryAccountManager.GetInstance ().AddAccount (account);
 			}
 			Dialog.Destroy ();
 		}
@@ -448,7 +474,7 @@ namespace FSpot {
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
 			if (gallery.Version == GalleryVersion.Version1) {
-				Gtk.MenuItem top_item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(TopLevel)"));
+				Gtk.MenuItem top_item = new Gtk.MenuItem (Catalog.GetString ("(TopLevel)"));
 				menu.Append (top_item);
 			}
 			
@@ -477,10 +503,18 @@ namespace FSpot {
 		
 		private void HandleChanged (object sender, EventArgs args)
 		{
-			if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
-				parent = "";
+			if (gallery.Version == GalleryVersion.Version1) {
+				if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
+					parent = "";
+				} else {
+					parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
+				}
 			} else {
-				parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
+				if (gallery.Albums.Count == 0 || album_optionmenu.History < 0) {
+					parent = "";
+				} else {
+					parent = ((Album) gallery.Albums [album_optionmenu.History]).Name;
+				}
 			}
 			name = name_entry.Text;
 			description = description_entry.Text;
@@ -538,11 +572,13 @@ namespace FSpot {
 			LoadPreference (Preferences.EXPORT_GALLERY_SIZE);
 			LoadPreference (Preferences.EXPORT_GALLERY_BROWSER);
 			LoadPreference (Preferences.EXPORT_GALLERY_META);
+			LoadPreference (Preferences.EXPORT_GALLERY_ROTATE);
 		}
 		
 		Gtk.ResponseHandler rh;
 		
 		private bool scale;
+		private bool rotate;
 		private int size;
 		private bool browser;
 		private bool meta;
@@ -572,6 +608,7 @@ namespace FSpot {
 		[Glade.Widget] Gtk.CheckButton browser_check;
 		[Glade.Widget] Gtk.CheckButton scale_check;
 		[Glade.Widget] Gtk.CheckButton meta_check;
+		[Glade.Widget] Gtk.CheckButton rotate_check;
 		
 		[Glade.Widget] Gtk.SpinButton size_spin;
 
@@ -602,6 +639,7 @@ namespace FSpot {
 
 			browser = browser_check.Active;
 			meta = meta_check.Active;
+			rotate = rotate_check.Active;
 
 			if (account != null) { 
 				//System.Console.WriteLine ("history = {0}", album_optionmenu.History);
@@ -611,7 +649,7 @@ namespace FSpot {
 				Dialog.Destroy ();
 
 				command_thread = new System.Threading.Thread (new System.Threading.ThreadStart (this.Upload));
-				command_thread.Name = Mono.Posix.Catalog.GetString ("Uploading Pictures");
+				command_thread.Name = Catalog.GetString ("Uploading Pictures");
 				
 				progress_dialog = new FSpot.ThreadProgressDialog (command_thread, photos.Length);
 				progress_dialog.Start ();
@@ -621,6 +659,7 @@ namespace FSpot {
 				Preferences.Set (Preferences.EXPORT_GALLERY_SIZE, size);
 				Preferences.Set (Preferences.EXPORT_GALLERY_BROWSER, browser);
 				Preferences.Set (Preferences.EXPORT_GALLERY_META, meta);
+				Preferences.Set (Preferences.EXPORT_GALLERY_ROTATE, rotate);
 			}
 		}
 		
@@ -642,38 +681,51 @@ namespace FSpot {
 				account.Gallery.Progress.Changed += HandleProgressChanged;
 
 				System.Console.WriteLine ("Starting upload");
+				
+				FilterSet filters = new FilterSet ();
+				if (scale)
+					filters.Add (new ResizeFilter ((uint)size));
+				else if (rotate)
+					filters.Add (new OrientationFilter ());
+				
+
 				while (photo_index < photos.Length) {
 					Photo photo = photos [photo_index];
 
 					System.Console.WriteLine ("uploading {0}", photo_index);
 
-					progress_dialog.Message = System.String.Format (Mono.Posix.Catalog.GetString ("Uploading picture \"{0}\""), photo.Name);
+					progress_dialog.Message = System.String.Format (Catalog.GetString ("Uploading picture \"{0}\""), photo.Name);
 					progress_dialog.Fraction = photo_index / (double) photos.Length;
 					photo_index++;
 
-					progress_dialog.ProgressText = System.String.Format (Mono.Posix.Catalog.GetString ("{0} of {1}"), photo_index, photos.Length);
+					progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), photo_index, photos.Length);
 					
-					if (scale) {
-						/* FIXME this is sick */
-						string orig = photo.DefaultVersionUri.LocalPath;
-						string path = PixbufUtils.Resize (orig, size, true);
-						string final = path + System.IO.Path.GetExtension (orig);
-						System.IO.File.Move (path, final);
-						album.Add (photo, final);
+					
+					string orig = photo.DefaultVersionUri.LocalPath;
+
+					// FIXME the filters need to be able to handle entension
+					// changes directly themselves.
+					string final = orig;
+					if (scale)
+						final = ImageFile.TempPath (orig, "jpg");
+					else if (rotate)
+						final = ImageFile.TempPath (orig);
+					
+					bool changed = filters.Convert (orig, final);
+					album.Add (photo, final);
+					
+					if (final != orig)
 						System.IO.File.Delete (final);
-					} else {
-						album.Add (photo);
-					}
 				}
 
-				progress_dialog.Message = Mono.Posix.Catalog.GetString ("Done Sending Photos");
+				progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
 				progress_dialog.Fraction = 1.0;
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Upload Complete");
+				progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
 				progress_dialog.ButtonLabel = Gtk.Stock.Ok;
 			} catch (System.Exception e) {
-				progress_dialog.Message = String.Format (Mono.Posix.Catalog.GetString ("Error Uploading To Gallery: {0}"),
+				progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To Gallery: {0}"),
 									 e.Message);
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error");
+				progress_dialog.ProgressText = Catalog.GetString ("Error");
 				System.Console.WriteLine (e);
 			}
 			
@@ -690,7 +742,7 @@ namespace FSpot {
 
 			accounts = manager.GetAccounts ();
 			if (accounts == null || accounts.Count == 0) {
-				Gtk.MenuItem item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(No Gallery)"));
+				Gtk.MenuItem item = new Gtk.MenuItem (Catalog.GetString ("(No Gallery)"));
 				menu.Append (item);
 				gallery_optionmenu.Sensitive = false;
 				edit_button.Sensitive = false;
@@ -782,8 +834,8 @@ namespace FSpot {
 			bool disconnected = gallery == null || !account.Connected || albums == null;
 
 			if (disconnected || albums.Count == 0) {
-				string msg = disconnected ? Mono.Posix.Catalog.GetString ("(Not Connected)") 
-					: Mono.Posix.Catalog.GetString ("(No Albums)");
+				string msg = disconnected ? Catalog.GetString ("(Not Connected)") 
+					: Catalog.GetString ("(No Albums)");
 
 				Gtk.MenuItem item = new Gtk.MenuItem (msg);
 				menu.Append (item);
@@ -868,6 +920,10 @@ namespace FSpot {
 				if (meta_check.Active != (bool) val)
 					meta_check.Active = (bool) val;
 				break;
+			case Preferences.EXPORT_GALLERY_ROTATE:
+				if (rotate_check.Active != (bool) val)
+					rotate_check.Active = (bool) val;
+				break;
 			}
 		}
 	}
