@@ -18,7 +18,7 @@ using LibGPhoto2;
 
 public class MainWindow {
 
-        public static MainWindow Toplevel;
+    public static MainWindow Toplevel;
 
 	Db db;
 
@@ -37,6 +37,8 @@ public class MainWindow {
 	[Glade.Widget] Notebook view_notebook;
 	[Glade.Widget] ScrolledWindow tag_selection_scrolled;
 
+	[Glade.Widget] Label status_label;
+
 	// File
 	[Glade.Widget] MenuItem version_menu_item;
 	[Glade.Widget] MenuItem create_version_menu_item;
@@ -82,7 +84,13 @@ public class MainWindow {
 	[Glade.Widget] CheckMenuItem reverse_order;
 
 	// Find
-	[Glade.Widget] MenuItem find_tag;
+	[Glade.Widget] MenuItem find_by_tag;
+	[Glade.Widget] MenuItem find_add_tag;
+	[Glade.Widget] MenuItem find_add_tag_with;
+	[Glade.Widget] MenuItem clear_tag_query;
+	
+	[Glade.Widget] MenuItem clear_date_range;
+
 	[Glade.Widget] CheckMenuItem find_untagged;
 	
 	// Tags
@@ -105,6 +113,8 @@ public class MainWindow {
 
 	Gtk.Toolbar toolbar;
 
+	FindBar find_bar;
+
 	PhotoVersionMenu versions_submenu;
 
 	Gtk.ToggleButton browse_button;
@@ -117,6 +127,7 @@ public class MainWindow {
 	FSpot.FullScreenView fsview;
 	FSpot.PhotoQuery query;
 	FSpot.GroupSelector group_selector;
+	FSpot.QueryWidget query_widget;
 	MainSelection selection;
 	
 	FSpot.Delay slide_delay;
@@ -129,15 +140,17 @@ public class MainWindow {
 	bool write_metadata = false;
 
 	// Drag and Drop
-	enum TargetType {
+	public enum TargetType {
 		UriList,
 		TagList,
+		TagQueryItem,
 		PhotoList,
 		RootWindow
 	};
 
 	private static TargetEntry [] icon_source_target_table = new TargetEntry [] {
 		new TargetEntry ("application/x-fspot-photos", 0, (uint) TargetType.PhotoList),
+		new TargetEntry ("application/x-fspot-tag-query-item", 0, (uint) TargetType.TagQueryItem),
 		new TargetEntry ("text/uri-list", 0, (uint) TargetType.UriList),
 		new TargetEntry ("application/x-root-window-drop", 0, (uint) TargetType.RootWindow)
 	};
@@ -181,6 +194,10 @@ public class MainWindow {
 		get { return selection; }
 	}
 
+	public MenuItem ClearFindByTag {
+		get { return clear_tag_query; }
+	}
+
 	//
 	// Constructor
 	//
@@ -188,6 +205,9 @@ public class MainWindow {
 	{
 		this.db = db;
 
+		if (Toplevel == null)
+			Toplevel = this;
+
 		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", "f-spot");
 		gui.Autoconnect (this);
 
@@ -242,6 +262,7 @@ public class MainWindow {
 
 		tag_selection_widget.ButtonPressEvent += HandleTagSelectionButtonPressEvent;
 		tag_selection_widget.PopupMenu += HandleTagSelectionPopupMenu;
+		tag_selection_widget.RowActivated += HandleTagSelectionRowActivated;
 		
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
@@ -255,10 +276,13 @@ public class MainWindow {
 			query = new FSpot.PhotoQuery (db.Photos);
 		}
 
+        UpdateStatusLabel ();
 		query.Changed += HandleQueryChanged;
 
 		db.Photos.ItemsChanged += HandleDbItemsChanged;
 		db.Tags.ItemsChanged += HandleTagsChanged;
+		db.Tags.ItemsAdded += HandleTagsChanged;
+		db.Tags.ItemsRemoved += HandleTagsChanged;
 #if SHOW_CALENDAR
 		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
 		cal.DaySelected += HandleCalendarDaySelected;
@@ -277,9 +301,14 @@ public class MainWindow {
 		view_vbox.PackStart (group_selector, false, false, 0);
 		view_vbox.ReorderChild (group_selector, 0);
 
-		FSpot.QueryDisplay query_display = new FSpot.QueryDisplay (query, tag_selection_widget);
-		view_vbox.PackStart (query_display, false, false, 0);
-		view_vbox.ReorderChild (query_display, 1);
+		find_bar = new FindBar (query, tag_selection_widget.Model);
+		view_vbox.PackStart (find_bar, false, false, 0);
+		main_window.KeyPressEvent += HandleKeyPressEvent;
+		
+		query_widget = new FSpot.QueryWidget (query, db, tag_selection_widget);
+		query_widget.Logic.Changed += HandleQueryLogicChanged;
+		view_vbox.PackStart (query_widget, false, false, 0);
+		view_vbox.ReorderChild (query_widget, 1);
 
 		icon_view = new QueryView (query);
 		icon_view.ZoomChanged += HandleZoomChanged;
@@ -299,16 +328,9 @@ public class MainWindow {
 		icon_view.DragBegin += HandleIconViewDragBegin;
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
-		TagMenu menu = new TagMenu (attach_tag, db.Tags);
-		menu.NewTagHandler += delegate { HandleCreateTagAndAttach (main_window, null); };
-		menu.TagSelected += HandleAttachTagMenuSelected;
-
 		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
 		far_image.SetFromStock ("f-spot-stock_far", IconSize.SmallToolbar);
 
-		menu = new TagMenu (find_tag, db.Tags);
-		menu.TagSelected += HandleFindTagMenuSelected;
-
 		PhotoTagMenu pmenu = new PhotoTagMenu ();
 		pmenu.TagSelected += HandleRemoveTagMenuSelected;
 		remove_tag.Submenu = pmenu;
@@ -322,7 +344,7 @@ public class MainWindow {
 		icon_view.DragDataReceived += HandleIconViewDragDataReceived;
 		icon_view.KeyPressEvent += HandleIconViewKeyPressEvent;
 
-		photo_view = new PhotoView (query, db.Photos);
+		photo_view = new PhotoView (query);
 		photo_box.Add (photo_view);
 		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
 		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
@@ -359,6 +381,7 @@ public class MainWindow {
 		main_window.ShowAll ();
 
 		tagbar.Hide ();
+		find_bar.Hide ();
 
 		LoadPreference (Preferences.SHOW_TOOLBAR);
 		LoadPreference (Preferences.SHOW_SIDEBAR);
@@ -368,10 +391,8 @@ public class MainWindow {
 
 		main_window.DeleteEvent += HandleDeleteEvent;
 		
-		query_display.HandleChanged (query);
-
-		if (Toplevel == null)
-			Toplevel = this;
+		query_widget.HandleChanged (query);
+		query_widget.Hide ();
 
 		// When the icon_view is loaded, set it's initial scroll position
 		icon_view.SizeAllocated += HandleIconViewReady;
@@ -415,6 +436,7 @@ public class MainWindow {
 			if (view_notebook.CurrentPage != 0)
 				view_notebook.CurrentPage = 0;
 				
+            ColorDialog.Close ();
 			JumpTo (photo_view.Item.Index);
 			zoom_scale.Value = icon_view.Zoom;
 			break;
@@ -469,6 +491,7 @@ public class MainWindow {
 
 	private void HandleTagsChanged (object sender, DbItemEventArgs args)
 	{
+		icon_view.QueueDraw ();
 		UpdateTagEntryFromSelection ();
 	}
 
@@ -640,6 +663,7 @@ public class MainWindow {
 	{
 		UpdateMenus ();
 		UpdateTagEntryFromSelection ();
+		info_box.Update ();
 	}
 
 
@@ -720,6 +744,13 @@ public class MainWindow {
 		}
 	}
 
+	void HandleTagSelectionRowActivated (object sender, RowActivatedArgs args)
+	{
+        ShowQueryWidget ();
+		//query_widget.Require (new Tag [] {tag_selection_widget.TagByPath (args.Path)});
+		query_widget.Include (new Tag [] {tag_selection_widget.TagByPath (args.Path)});
+	}
+
 	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
 		if (args.Event.Button == 3)
@@ -742,8 +773,8 @@ public class MainWindow {
 	{
 		string backup = db.Repair ();
 		string short_msg = Mono.Posix.Catalog.GetString ("Error loading database.");
-		string long_msg = Mono.Posix.Catalog.GetString ("F-Spot encountered an error while loading the photo database" + 
-								"  the old database has be moved to {0} and a new database has been created.");
+		string long_msg = Mono.Posix.Catalog.GetString ("F-Spot encountered an error while loading the photo database. " + 
+								"The old database has be moved to {0} and a new database has been created.");
 
 		HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
 							    MessageType.Error, ButtonsType.Ok, 
@@ -810,6 +841,12 @@ public class MainWindow {
 			return;
 
 		tag_selection_widget.SetDragDestRow (path, Gtk.TreeViewDropPosition.IntoOrAfter);
+
+		//Scroll if required
+		if (args.Y < 20)
+			tag_selection_scrolled.Vadjustment.Value -= 30;
+		if (((o as Gtk.Widget).Allocation.Height - args.Y) < 20)
+			tag_selection_scrolled.Vadjustment.Value += 30;
 	}
 
 	public void HandleTagSelectionDragDataReceived (object o, DragDataReceivedArgs args)
@@ -829,6 +866,7 @@ public class MainWindow {
 				AddTagExtended (num, tags);
 			}
 			db.CommitTransaction ();
+			query_widget.PhotoTagsChanged (tags);
 			break;
 		case (uint)TargetType.UriList:
 			UriList list = new UriList (args.SelectionData);
@@ -839,7 +877,7 @@ public class MainWindow {
 				
 				// FIXME - at this point we should import the photo, and then continue
 				if (photo == null)
-					return;
+					continue;
 				
 				// FIXME this should really follow the AddTagsExtended path too
 				photo.AddTag (tags);
@@ -848,6 +886,9 @@ public class MainWindow {
 			InvalidateViews ();
 			break;
 		case (uint)TargetType.TagList:
+			if (tag_selection_widget.TagHighlight.Length < 1)
+				return;
+
 			Tag child = tag_selection_widget.TagHighlight [0];
 			Tag parent = tags[0];
 
@@ -1227,12 +1268,19 @@ public class MainWindow {
 			AddTagExtended (num, new Tag [] {t});
 		}
 		db.CommitTransaction ();
+		query_widget.PhotoTagsChanged (new Tag[] {t});
 	}
 	
-	void HandleFindTagMenuSelected (Tag t)
+	public void HandleRequireTag (object sender, EventArgs args)
+ 	{
+        ShowQueryWidget ();
+		query_widget.Require (tag_selection_widget.TagHighlight);
+ 	}
+ 
+	public void HandleUnRequireTag (object sender, EventArgs args)
 	{
-		tag_selection_widget.TagSelection = new Tag [] {t};
-	}
+		query_widget.UnRequire (tag_selection_widget.TagHighlight);
+ 	}
 
 	public void HandleRemoveTagMenuSelected (Tag t)
 	{
@@ -1242,6 +1290,7 @@ public class MainWindow {
 			query.Commit (num);
 		}
 		db.CommitTransaction ();
+		query_widget.PhotoTagsChanged (new Tag [] {t});
 	}
 
 	//
@@ -1370,6 +1419,10 @@ public class MainWindow {
 		info_display_window.Destroyed += HandleInfoDisplayDestroy;
 	}
 
+	void HandleExportToPicasa (object sender, EventArgs args)
+	{
+		new FSpot.GoogleExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
 
 	void HandleExportToGallery (object sender, EventArgs args)
 	{
@@ -1422,12 +1475,7 @@ public class MainWindow {
 
 	void HandleSendMailCommand (object sender, EventArgs args)
 	{
-		StringBuilder url = new StringBuilder ("mailto:?subject=my%20photos");
-
-		foreach (Photo p in SelectedPhotos ()) {
-			url.Append ("&attach=" + UriList.EscapeString (p.DefaultVersionPath, true, true, true));
-		}
-		GnomeUtil.UrlShow (main_window, url.ToString ());
+		new FSpot.SendEmail (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
 	public static void HandleAbout (object sender, EventArgs args)
@@ -1495,6 +1543,10 @@ public class MainWindow {
 
 		if (sender != month)
 			month.Active = true;
+
+		//update the selection in the Timeline
+		if (query.Range != null)
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
 	}
 
 	public void HandleArrangeByDirectory (object sender, EventArgs args)
@@ -1527,6 +1579,13 @@ public class MainWindow {
 		// FIXME this is blah...we need UIManager love here
 		if (item != reverse_order)
 			reverse_order.Active = item.Active;
+		
+		//update the selection in the timeline
+		if ( query.Range != null && group_selector.Adaptor is TimeAdaptor) {
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
+			
+		}
+
 	}
 
 	// Called when the user clicks the X button	
@@ -1600,7 +1659,7 @@ public class MainWindow {
 	        long length = 0;
 
 		foreach (Photo p in photos) {
-			System.IO.FileInfo fi = new System.IO.FileInfo (p.DefaultVersionPath);
+			System.IO.FileInfo fi = new System.IO.FileInfo (p.DefaultVersionUri.LocalPath);
 
 			length += fi.Length;
 		}
@@ -1653,6 +1712,7 @@ public class MainWindow {
 			AddTagExtended (num, tags);
 		}
 		db.CommitTransaction ();
+		query_widget.PhotoTagsChanged (tags);
 	}
 
 	public void HandleRemoveTagCommand (object obj, EventArgs args)
@@ -1668,6 +1728,7 @@ public class MainWindow {
 			query.Commit (num);
 		}
 		db.CommitTransaction ();
+		query_widget.PhotoTagsChanged (tags);
 	}
 
 	public void HandleEditSelectedTag (object sender, EventArgs ea)
@@ -1686,8 +1747,6 @@ public class MainWindow {
 		
 		TagCommands.Edit command = new TagCommands.Edit (db, main_window);
 		command.Execute (tag);
-		if (view_mode == ModeType.IconView)
-			icon_view.QueueDraw ();
 	}
 
 	public void HandleMergeTagsCommand (object obj, EventArgs args)
@@ -1755,10 +1814,6 @@ public class MainWindow {
 		db.Photos.Remove (removetags);
 		db.CommitTransaction ();
 
-		UpdateTagEntryFromSelection ();
-
-		icon_view.QueueDraw ();
-
 		HandleEditSelectedTagWithTag (survivor);
 	}
 
@@ -1773,7 +1828,7 @@ public class MainWindow {
 	{
 		if (ActiveIndex () >= 0) {
 			SetViewMode (ModeType.PhotoView);
-			new FSpot.ColorDialog (photo_view.View);
+			ColorDialog.CreateForView (photo_view.View);
 		}
 	}
 
@@ -1945,9 +2000,12 @@ public class MainWindow {
 		if (fsview == null) {
 			fsview = new FSpot.FullScreenView (query);
 			fsview.Destroyed += HandleFullScreenViewDestroy;
-		}
-		// FIXME this needs to be another mode like PhotoView and IconView mode.
-		fsview.View.Item.Index = active;
+            fsview.View.Item.Index = active;
+            ColorDialog.SwitchViews (fsview.View);
+		} else {
+            // FIXME this needs to be another mode like PhotoView and IconView mode.
+            fsview.View.Item.Index = active;
+        }
 
 		fsview.Show ();
 	}
@@ -1956,6 +2014,10 @@ public class MainWindow {
 	{
 		JumpTo (fsview.View.Item.Index);
 		fsview = null;
+
+        if (ViewMode == ModeType.PhotoView) {
+            ColorDialog.SwitchViews (photo_view.View);
+        }
 	}
 	
 	void HandleZoomScaleValueChanged (object sender, System.EventArgs args)
@@ -1981,6 +2043,15 @@ public class MainWindow {
 	{
 		if (find_untagged.Active != query.Untagged)
 			find_untagged.Active = query.Untagged;
+
+		
+		clear_date_range.Sensitive = (query.Range != null);
+		UpdateStatusLabel ();
+	}
+
+	private void UpdateStatusLabel ()
+	{
+		status_label.Text = String.Format (Catalog.GetPluralString ("{0} Photo", "{0} Photos", query.Count), query.Count);
 	}
 	
 	void HandleZoomChanged (object sender, System.EventArgs args)
@@ -2007,11 +2078,21 @@ public class MainWindow {
 		zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
 	}
 
+	void HandleZoomOut (object sender, ButtonPressEventArgs args)
+	{
+		ZoomOut ();
+	}
+	
 	void HandleZoomOut (object sender, EventArgs args)
 	{
 		ZoomOut ();
 	}
 	
+	void HandleZoomIn (object sender, ButtonPressEventArgs args)
+	{
+		ZoomIn ();
+	}
+	
 	void HandleZoomIn (object sender, EventArgs args)
 	{
 		ZoomIn ();
@@ -2103,11 +2184,11 @@ public class MainWindow {
 			
 			foreach (Photo photo in photos) {
 				foreach (uint id in photo.VersionIds) {
-					Console.WriteLine (" path == {0}", photo.GetVersionPath (id));
+					Console.WriteLine (" path == {0}", photo.VersionUri (id).LocalPath);
 					try {
 						photo.DeleteVersion (id, true);
 					} catch (Exception e) {
-						DeleteException (e, photo.GetVersionPath (id));
+						DeleteException (e, photo.VersionUri (id).ToString ());
 					}
 				}
 			}
@@ -2214,7 +2295,6 @@ public class MainWindow {
 				md.Destroy ();
 			}
 		}
-		icon_view.QueueDraw ();
 	}
 
 	void HandleUpdateThumbnailCommand (object sende, EventArgs args)
@@ -2230,11 +2310,23 @@ public class MainWindow {
 
 	public void HandleRotate90Command (object sender, EventArgs args)
 	{
+        // Don't steal characters from any text entries
+        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+            return;
+        }
+
 		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Clockwise);
 	}
 
 	public void HandleRotate270Command (object sender, EventArgs args)
 	{
+        // Don't steal characters from any text entries
+        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+            return;
+        }
+
 		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Counterclockwise);
 	}
 
@@ -2253,7 +2345,7 @@ public class MainWindow {
 			if (i++ > 0)
 				paths.Append (" ");
 
-			paths.Append (System.IO.Path.GetFullPath (p.DefaultVersionPath));
+			paths.Append (System.IO.Path.GetFullPath (p.DefaultVersionUri.LocalPath));
 		}
 		
 		String data = paths.ToString ();
@@ -2273,16 +2365,22 @@ public class MainWindow {
 		client.Set ("/desktop/gnome/background/primary_color", "#000000");
 		client.Set ("/desktop/gnome/background/picture_options", "scaled");
 		client.Set ("/desktop/gnome/background/picture_opacity", 100);
-		client.Set ("/desktop/gnome/background/picture_filename", current.DefaultVersionPath);
+		client.Set ("/desktop/gnome/background/picture_filename", current.DefaultVersionUri.LocalPath);
 		client.Set ("/desktop/gnome/background/draw_background", true);
 	}
 
 	void HandleSetDateRange (object sender, EventArgs args) {
 		DateCommands.Set set_command = new DateCommands.Set (query, main_window);
 		set_command.Execute ();
+		//update the TimeLine
+		if (group_selector.Adaptor is TimeAdaptor && query.Range != null) 
+			group_selector.SetLimitsToDates(query.Range.Start, query.Range.End);
 	}
 
-	void HandleClearDateRange (object sender, EventArgs args) {
+	public void HandleClearDateRange (object sender, EventArgs args) {
+		if (group_selector.Adaptor is FSpot.TimeAdaptor) {
+			group_selector.ResetLimits();
+		}
 		query.Range = null;
 	}
 	
@@ -2443,13 +2541,99 @@ public class MainWindow {
 	{
 		UpdateMenus ();
 	}
+
+	public bool TagIncluded (Tag tag)
+	{
+		return query_widget.TagIncluded (tag);
+	}
+	
+	public bool TagRequired (Tag tag)
+	{
+		return query_widget.TagRequired (tag);
+	}
+
+    private void HandleQueryLogicChanged (object sender, EventArgs args)
+    {
+        HandleFindAddTagWith (null, null);
+    }
+
+	public void HandleIncludeTag (object sender, EventArgs args)
+	{
+        ShowQueryWidget ();
+		query_widget.Include (tag_selection_widget.TagHighlight);
+	}
+	
+	public void HandleUnIncludeTag (object sender, EventArgs args)
+	{
+		query_widget.UnInclude (tag_selection_widget.TagHighlight);
+ 	}
+
+    void HandleFindByTag (object sender, EventArgs args)
+    {
+        ShowQueryWidget ();
+    }
+
+    void HandleFindAddTagWith (object sender, EventArgs args)
+    {
+        if (find_add_tag_with.Submenu != null)
+           find_add_tag_with.Submenu.Dispose ();
+
+        Gtk.Menu submenu = FSpot.Query.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
+        if (submenu == null)
+            find_add_tag_with.RemoveSubmenu();
+        else
+            find_add_tag_with.Submenu = submenu;
+
+        find_add_tag_with.Sensitive = (submenu != null);
+    }
+
+    public void HandleAddTagToTerm (object sender, EventArgs args)
+    {
+        MenuItem item = sender as MenuItem;
+
+        int item_pos = 0;
+        foreach (MenuItem i in (item.Parent as Menu).Children) {
+            if (item == i) {
+                break;
+            }
+
+            item_pos++;
+        }
+        // account for All and separator menu items
+        item_pos -= 2;
+
+        FSpot.Query.LogicTerm parent_term = (FSpot.Query.LogicTerm) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
+
+        if (FSpot.Query.LogicWidget.Box != null) {
+            FSpot.Query.Literal after = parent_term.Last as FSpot.Query.Literal;
+            FSpot.Query.LogicWidget.Box.InsertTerm (tag_selection_widget.TagHighlight, parent_term, after);
+        }
+    }
+
+	void HandleFindTagIncluded (Tag t)
+	{
+        ShowQueryWidget ();
+		query_widget.Include (new Tag [] {t});
+ 	}
+	
+	void HandleFindTagRequired (Tag t)
+	{
+        ShowQueryWidget ();
+		query_widget.Require (new Tag [] {t});
+    }
+
+	void HandleClearTagSearch (object sender, EventArgs args)
+	{
+		query_widget.ClearTags ();
+	}
 	
 	//
 	// Handle Main Menu 
 
 	void UpdateMenus ()
 	{
-		bool tag_sensitive = tag_selection_widget.Selection.CountSelectedRows () > 0;
+		int tags_selected = tag_selection_widget.Selection.CountSelectedRows ();
+		bool tag_sensitive = tags_selected > 0;
 		bool active_selection = selection.Count > 0;
 		bool single_active = CurrentPhoto != null;
 		
@@ -2531,6 +2715,21 @@ public class MainWindow {
 				SetTip (rr_button, String.Format (msg, selection.Count));
 			}
 		}
+
+        //if (last_tags_selected_count != tags_selected) {
+            ((Gtk.Label)find_add_tag.Child).Text = String.Format (
+                Catalog.GetPluralString ("Find Selected Tag", "Find Selected Tags", tags_selected), tags_selected
+            );
+
+            ((Gtk.Label)find_add_tag_with.Child).Text = String.Format (
+                Catalog.GetPluralString ("Find Selected Tag With", "Find Selected Tags With", tags_selected), tags_selected
+            );
+
+            find_add_tag.Sensitive = tag_sensitive;
+            find_add_tag_with.Sensitive = tag_sensitive && find_add_tag_with.Submenu != null;
+
+            //last_tags_selected_count = tags_selected;
+        //}
 	}
 
 	public void HandleOpenWith (object sender, Gnome.Vfs.MimeApplication mime_application)
@@ -2571,25 +2770,26 @@ public class MainWindow {
 		ArrayList errors = new ArrayList ();
 		GLib.List uri_list = new GLib.List (typeof (string));
 		foreach (Photo photo in selected) {
-			// FIXME need to clean up the error dialog here.
 			try {
 				if (create_new_versions) {
 					uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
 					photo.DefaultVersionId = version;
 				}
-
-				uri_list.Append (photo.DefaultVersionPath);
+				query.MarkChanged (query.IndexOf (photo));
 			} catch (Exception e) {
 				errors.Add (new EditException (photo, e));
 			}
+
+			uri_list.Append (photo.DefaultVersionUri.ToString ());
 		}
 
+		// FIXME need to clean up the error dialog here.
 		if (errors.Count > 0) {
 			Dialog md = new EditExceptionDialog (GetToplevel (sender), errors.ToArray (typeof (EditException)) as EditException []);
 			md.Run ();
 			md.Destroy ();
 		}
-		
+
 		if (create_new_versions) {
 			db.Photos.Commit (selected, new DbItemEventArgs (selected));
 		}
@@ -2597,6 +2797,14 @@ public class MainWindow {
 		mime_application.Launch (uri_list);
 	}
 
+	public void GetWidgetPosition(Widget widget, out int x, out int y)
+    {
+		main_window.GdkWindow.GetOrigin(out x, out y);
+		
+		x += widget.Allocation.X;
+		y += widget.Allocation.Y;
+ 	}
+
 	// Tag typing
 
 	private ArrayList selected_photos_tagnames;
@@ -2661,15 +2869,14 @@ public class MainWindow {
 			return;
 #endif
 		
-		if (tag_entry.Text.Length > 0)
-			tag_entry.Text += ", ";
-
 #if ALLOW_TAG_TYPING_WITHOUT_HOTKEY
 		tag_entry.Text += c;
 #endif
 
 		tagbar.Show ();
 		UpdateTagEntryFromSelection ();
+		if (tag_entry.Text.Length > 0)
+			tag_entry.Text += ", ";
 		tag_entry.GrabFocus ();
 		tag_entry.SelectRegion (-1, -1);
 	}
@@ -2739,7 +2946,6 @@ public class MainWindow {
 	       }
 
 	       if (view_mode == ModeType.IconView) {
-		       icon_view.QueueDraw ();
 		       icon_view.GrabFocus ();
 	       } else {
 		       photo_view.QueueDraw ();
@@ -2876,7 +3082,7 @@ public class MainWindow {
 		ArrayList mimes = new ArrayList ();
 
 		foreach (Photo p in SelectedPhotos ()) {
-			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionPath);
+			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionUri.ToString ());
 
 			if (! mimes.Contains (mime))
 				mimes.Add (mime);
@@ -2885,6 +3091,37 @@ public class MainWindow {
 		return mimes.ToArray (typeof (string)) as string [];
 	}
 
+    private void ShowQueryWidget () {
+        if (find_bar.Visible) {
+            find_bar.Entry.Text = "";
+            find_bar.Hide ();
+        }
+
+        query_widget.Show ();
+        return;
+    }
+
+	public void HandleKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
+    {
+		bool ctrl = ModifierType.ControlMask == (args.Event.State & ModifierType.ControlMask);
+
+        if ((ctrl && args.Event.Key == Gdk.Key.F) || args.Event.Key == Gdk.Key.slash) {
+            if (!find_bar.Visible) {
+                if (query_widget.Visible) {
+                    query_widget.ClearTags ();
+                    query_widget.Hide ();
+                }
+
+                find_bar.Show ();
+                find_bar.Entry.GrabFocus ();
+                args.RetVal = true;
+                return;
+            }
+        }
+
+        args.RetVal = false;
+    }
+
 	public static void SetTip (Widget widget, string tip)
 	{
 		toolTips.SetTip (widget, tip, null);
