@@ -70,14 +70,27 @@ namespace FSpot
 		public const string EXPORT_GALLERY_SIZE = "/apps/f-spot/export/gallery/size";
 		public const string EXPORT_GALLERY_BROWSER = "/apps/f-spot/export/gallery/browser";
 		public const string EXPORT_GALLERY_META = "/apps/f-spot/export/gallery/meta";
+		public const string EXPORT_GALLERY_ROTATE = "/apps/f-spot/export/gallery/rotate";
+
+		public const string EXPORT_PICASAWEB_SCALE = "/apps/f-spot/export/picasaweb/scale";
+		public const string EXPORT_PICASAWEB_SIZE = "/apps/f-spot/export/picasaweb/size";
+		public const string EXPORT_PICASAWEB_ROTATE = "/apps/f-spot/export/picasaweb/rotate";
+		public const string EXPORT_PICASAWEB_BROWSER = "/apps/f-spot/export/picasaweb/browser";
+
+ 		public const string EXPORT_EMAIL_SIZE = "/apps/f-spot/export/email/size";
+ 		public const string EXPORT_EMAIL_ROTATE = "/apps/f-spot/export/email/auto_rotate";
+ 		public const string EXPORT_EMAIL_DELETE_TIMEOUT_SEC = "/apps/f-spot/export/email/delete_timeout_seconds";
 
 		public const string SCREENSAVER_TAG = "/apps/f-spot/screensaver/tag_id";
 
+		public const string STORAGE_PATH = "/apps/f-spot/import/storage_path";
+
 		public const string METADATA_EMBED_IN_IMAGE = "/apps/f-spot/metadata/embed_in_image";
 
 		public const string GNOME_SCREENSAVER_THEME = "/apps/gnome-screensaver/themes";
 		public const string GNOME_SCREENSAVER_MODE = "/apps/gnome-screensaver/mode";
 
+
 		static GConf.Client client;
 		static GConf.NotifyEventHandler changed_handler;
 
@@ -119,13 +132,28 @@ namespace FSpot
 			case SHOW_TIMELINE:
 			case SHOW_TAGS:
 			case SHOW_DATES:
+			case EXPORT_PICASAWEB_SCALE:
+			case EXPORT_PICASAWEB_ROTATE:
+			case EXPORT_PICASAWEB_BROWSER:
 				return true;
+			
+			case EXPORT_PICASAWEB_SIZE:
+				return 800;
 		
 			case SIDEBAR_POSITION:
 			case ZOOM:
 				return null;
+
 			case SCREENSAVER_TAG:
 				return 1;
+			case STORAGE_PATH:
+				return System.IO.Path.Combine (FSpot.Global.HomeDirectory, Catalog.GetString("Photos"));
+			case EXPORT_EMAIL_SIZE:
+				return 3;	// medium size 640px
+			case EXPORT_EMAIL_ROTATE:
+				return true;	// auto rotate original photos
+			case EXPORT_EMAIL_DELETE_TIMEOUT_SEC:
+				return 30;	// delete temporary email pictures after 30 seconds
 			}
 
 			return null;
