@@ -77,15 +77,10 @@ namespace FSpot {
 			Gnome.PrintContext ctx = print_job.Context;
 
 			foreach (Photo photo in photos) {
-				Gdk.Pixbuf image = null;
-				try {
-					image = FSpot.PhotoLoader.Load (photo);
-					Gnome.Print.Beginpage (ctx, "F-Spot" + photo.DefaultVersionPath);				
-				} catch (System.Exception e) {
-					System.Console.WriteLine (e);
-					continue;
-				}
-				
+				Gnome.Print.Beginpage (ctx, "F-Spot" + photo.DefaultVersionUri.ToString ());				
+
+				Gdk.Pixbuf image = FSpot.PhotoLoader.Load (photo);
+
 				bool rotate = false;
 				double width = page_width;
 				double height = page_height;
@@ -130,6 +125,8 @@ namespace FSpot {
 			Gnome.PrintDialog gnome_dialog = new Gnome.PrintDialog (print_job, "Print Photos", 0);
 			int response = gnome_dialog.Run ();
 			
+			Render ();
+
 			switch (response) {
 			case (int) Gnome.PrintButtons.Print:
 				print_job.Print ();
@@ -152,7 +149,7 @@ namespace FSpot {
 
 			print_job = new Gnome.PrintJob (Gnome.PrintConfig.Default ());
 
-			Render ();
+			//Render ();
 
 #if ENABLE_CUSTOM_PRINT
 			int response = print_dialog.Run ();
