@@ -1,7 +1,21 @@
+/*
+ * Filters/IFilter.cs
+ *
+ * Author(s)
+ *   Larry Ewing <lewing@novell.com>
+ *
+ * This is free software. See COPYING for details
+ *
+ */
+
 using System;
 using System.IO;
+
 using Gtk;
+using Gdk;
+
 using FSpot;
+using FSpot.Png;
 
 namespace FSpot {
 	public class RotateException : ApplicationException {
@@ -42,24 +56,46 @@ namespace FSpot {
 					     direction == RotateDirection.Clockwise ? JpegUtils.TransformType.Rotate90 
 					     : JpegUtils.TransformType.Rotate270);
 			
-			// FIXME way to do this atomically in .NET?  I think Move() raises an exception
-			// if the destination path points to an existing file.
-			File.Delete (original_path);
-			File.Move (temporary_path, original_path);
+			Unix.Rename (temporary_path, original_path);
 		}
 
 		private static void RotateOrientation (string original_path, RotateDirection direction)
 		{
 			FSpot.ImageFile img = FSpot.ImageFile.Create (original_path);
-			FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
-			if (jimg != null) {
+			if (img is JpegFile) {
+				FSpot.JpegFile jimg = img as FSpot.JpegFile;
 				PixbufOrientation orientation = direction == RotateDirection.Clockwise
 					? PixbufUtils.Rotate90 (img.Orientation)
 					: PixbufUtils.Rotate270 (img.Orientation);
 				
 				jimg.SetOrientation (orientation);
 				jimg.SaveMetaData (original_path);
+			} else if (img is PngFile) {
+				PngFile png = img as PngFile;
+				bool supported = false;
+
+				foreach (PngFile.Chunk c in png.Chunks) {
+					PngFile.IhdrChunk ihdr = c as PngFile.IhdrChunk;
+					
+					if (ihdr != null && ihdr.Depth == 8)
+						supported = true;
+				}
+
+				if (! supported)
+					throw new RotateException ("Unable to rotate photo type", original_path);
+
+				string backup = ImageFile.TempPath (original_path);
+				using (Stream stream = File.Open (backup, FileMode.Truncate, FileAccess.Write)) {
+					using (Pixbuf pixbuf = img.Load ()) {
+						PixbufOrientation fake = (direction == RotateDirection.Clockwise) ? PixbufOrientation.RightTop : PixbufOrientation.LeftBottom;
+						using (Pixbuf rotated = PixbufUtils.TransformOrientation (pixbuf, fake)) {
+							img.Save (rotated, stream);
+						}
+					}
+				}
+				File.Copy (backup, original_path, true);
+				File.Delete (backup);
 			} else {
 				throw new RotateException ("Unable to rotate photo type", original_path);
 			}
