@@ -39,6 +39,9 @@ public class TagSelectionWidget : TreeView {
 	// FIXME this is a hack.
 	private static Pixbuf empty_pixbuf = new Pixbuf (Colorspace.Rgb, true, 8, 1, 1);
 
+	// If these are changed, the base () call in the constructor must be updated.
+	private const int IdColumn = 0;
+	private const int NameColumn = 1;
 
 	// Selection management.
 
@@ -48,23 +51,33 @@ public class TagSelectionWidget : TreeView {
 	public Tag TagAtPosition (int x, int y) 
 	{
 		TreePath path;
-		TreeIter iter;
 
 		// Work out which tag we're dropping onto
 		if (!this.GetPathAtPos (x, y, out path))
 			return null;
 
+		return TagByPath (path);
+	}
+
+	public Tag TagByPath (TreePath path) 
+	{
+		TreeIter iter;
+
 		if (!Model.GetIter (out iter, path))
 			return null;
 
-		GLib.Value value = new GLib.Value ();
-
-		Model.GetValue (iter, 0, ref value);
-		uint tag_id = (uint) value;
-		Tag tag = tag_store.Get (tag_id) as Tag;
-
-		return tag;
+		return TagByIter (iter);
 	}
+	
+	public Tag TagByIter (TreeIter iter)
+	{
+		GLib.Value val = new GLib.Value ();
+ 
+		Model.GetValue (iter, IdColumn, ref val);
+		uint tag_id = (uint) val;
+ 
+		return tag_store.Get (tag_id) as Tag;
+ 	}
 
 	public void Select (Tag tag)
 	{
@@ -129,8 +142,7 @@ public class TagSelectionWidget : TreeView {
 			foreach (Tag t in value)
 				Select (t);
 
-			//FIXME this should really just toggle the items not rebuild the list
-			Update ();
+			QueueDraw ();
 
 			if (SelectionChanged != null)
 				SelectionChanged (this);
@@ -151,24 +163,49 @@ public class TagSelectionWidget : TreeView {
 		}
 	}
 
-	public Tag [] TagHighlight () {
-		TreeModel model;
+	public void ScrollTo (Tag tag)
+	{
 		TreeIter iter;
+		if (! TreeIterForTag (tag, out iter))
+			return;
 
-		TreePath [] rows = Selection.GetSelectedRows(out model);
+		TreePath path = Model.GetPath (iter);
 
-		Tag [] tags = new Tag [rows.Length];
-		int i = 0;
+		ScrollToCell (path, null, false, 0, 0);
+	}
 
-		foreach (TreePath path in rows) {
-			GLib.Value value = new GLib.Value ();
-			Model.GetIter (out iter, path);
-			Model.GetValue (iter, 0, ref value);
-			uint tag_id = (uint) value;
-			tags[i] = tag_store.Get (tag_id) as Tag;
-			i++;
+	public Tag [] TagHighlight {
+		get {
+			TreeModel model;
+			TreeIter iter;
+
+			TreePath [] rows = Selection.GetSelectedRows(out model);
+
+			Tag [] tags = new Tag [rows.Length];
+			int i = 0;
+
+			foreach (TreePath path in rows) {
+				GLib.Value value = new GLib.Value ();
+				Model.GetIter (out iter, path);
+				Model.GetValue (iter, IdColumn, ref value);
+				uint tag_id = (uint) value;
+				tags[i] = tag_store.Get (tag_id) as Tag;
+				i++;
+			}
+			return tags;
+		}
+
+		set {
+			if (value == null)
+				return;
+
+			Selection.UnselectAll ();
+
+			TreeIter iter;
+			foreach (Tag tag in value)
+				if (TreeIterForTag (tag, out iter))
+					Selection.SelectIter (iter);
 		}
-		return tags;
 	}
 
 	public void Update ()
@@ -197,7 +234,7 @@ public class TagSelectionWidget : TreeView {
 		Model.GetIter (out iter, path);
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, ref value);
+		Model.GetValue (iter, IdColumn, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -234,10 +271,12 @@ public class TagSelectionWidget : TreeView {
 	{
 		// FIXME this should be themable but Gtk# doesn't give me access to the proper
 		// members in GtkStyle for that.
+		/*
 		if (tag is Category)
 			renderer.CellBackground = ToHashColor (this.Style.MidColors [(int) Gtk.StateType.Normal]);
 		else
 			renderer.CellBackground = ToHashColor (this.Style.LightColors [(int) Gtk.StateType.Normal]);
+		*/
 	}
 
 	private void CheckBoxDataFunc (TreeViewColumn column,
@@ -246,7 +285,7 @@ public class TagSelectionWidget : TreeView {
 				       TreeIter iter)
 	{
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, ref value);
+		Model.GetValue (iter, IdColumn, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -260,7 +299,7 @@ public class TagSelectionWidget : TreeView {
 				   TreeIter iter)
 	{
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, ref value);
+		Model.GetValue (iter, IdColumn, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -283,7 +322,7 @@ public class TagSelectionWidget : TreeView {
 			return;
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, ref value);
+		Model.GetValue (iter, IdColumn, ref value);
 		uint tag_id = (uint) value;
 
 		Tag tag = tag_store.Get (tag_id) as Tag;
@@ -321,7 +360,7 @@ public class TagSelectionWidget : TreeView {
 		}
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (parent, 0, ref value);
+		Model.GetValue (parent, IdColumn, ref value);
 		iter = parent;
 
 		if (tag.Id == (uint) value)
@@ -339,7 +378,7 @@ public class TagSelectionWidget : TreeView {
 		TreeStore store = Model as TreeStore;
 		bool valid;
 		
-		store.GetValue (src, 0, ref value);
+		store.GetValue (src, IdColumn, ref value);
 		Tag tag = (Tag) tag_store.Get ((uint)value);
 		if (is_parent) {
 			// we need to figure out where to insert it in the correct order
@@ -366,35 +405,38 @@ public class TagSelectionWidget : TreeView {
 
 		if (is_root)
 			valid = store.GetIterFirst (out iter);
-		else {
+		else
 			valid = store.IterChildren (out iter, parent);
-			ExpandRow (Model.GetPath (parent), false);
-		}
 
 		while (valid) {
 			//I have no desire to figure out a more performant sort over this...
 			GLib.Value value = new GLib.Value ();
-			store.GetValue(iter, 0, ref value);
-			compare = (Tag)tag_store.Get ((uint) value);
+			store.GetValue(iter, IdColumn, ref value);
+			compare = (Tag) tag_store.Get ((uint) value);
 
 			if (compare.CompareTo (tag) > 0) {
 				iter = store.InsertNodeBefore (iter);
-				store.SetValue (iter, 0, tag.Id);
-				store.SetValue (iter, 1, tag.Name);
+				store.SetValue (iter, IdColumn, tag.Id);
+				store.SetValue (iter, NameColumn, tag.Name);
+				
+				if (!is_root)
+					ExpandRow (Model.GetPath (parent), false);
 				return iter;
 			}
 			valid = store.IterNext(ref iter);
 		}
-
+		
 		if (is_root) 
 			iter = store.AppendNode (); 
-		else
+		else {
 			iter = store.AppendNode (parent); 
+			ExpandRow (Model.GetPath (parent), false);
+		}
 
-		store.SetValue (iter, 0, tag.Id);
-		store.SetValue (iter, 1, tag.Name);
+		store.SetValue (iter, IdColumn, tag.Id);
+		store.SetValue (iter, NameColumn, tag.Name);
 		return iter;
-	}	
+	}
 
 	private void HandleTagsRemoved (object sender, DbItemEventArgs args)
 	{
@@ -467,7 +509,7 @@ public class TagSelectionWidget : TreeView {
 			foreach (TreeIter iter in iters)
 			{
 				GLib.Value v = new GLib.Value ();
-				Model.GetValue (iter, 0, ref v);
+				Model.GetValue (iter, IdColumn, ref v);
 				int tag_id = (int)(uint) v;
 				if (expanded_tags.Contains (tag_id)) {
 					ExpandRow (Model.GetPath (iter), false);
@@ -523,7 +565,7 @@ public class TagSelectionWidget : TreeView {
 		{
 			if (GetRowExpanded (Model.GetPath (iter))) {
 				GLib.Value v = new GLib.Value ();
-				Model.GetValue (iter, 0, ref v);
+				Model.GetValue (iter, IdColumn, ref v);
 				expanded_tags.Add ((int)(uint) v);
 			}
 		}
@@ -539,7 +581,21 @@ public class TagSelectionWidget : TreeView {
 		if (rows.Length != 1)
 			return;
 		
-		SetCursor (rows[0], name_column, true);
+		//SetCursor (rows[0], NameColumn, true);
+		text_render.Editable = true;
+		text_render.Edited += HandleTagNameEdited;
+		SetCursor (rows[0], complete_column, true);
+		text_render.Editable = false;
+	}
+
+	// Override the default ButtonPressEvent because it is stupid and
+	// returns true, meaning if you want to have another handler for the ButtonPress
+	// event you have to Glib.ConnectBefore.  Plus you usually want the default one to
+	// run first so it will select/unselect anything necessary before popping up a menu, say.
+	protected override bool OnButtonPressEvent (Gdk.EventButton button)
+	{
+		base.OnButtonPressEvent (button);
+		return false;
 	}
 	
 	public void HandleTagNameEdited (object sender, EditedArgs args)
@@ -552,7 +608,7 @@ public class TagSelectionWidget : TreeView {
 			return;
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, ref value);
+		Model.GetValue (iter, IdColumn, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -562,7 +618,7 @@ public class TagSelectionWidget : TreeView {
 
 		// Check that the tag doesn't already exist
 		if (String.Compare (args.NewText, tag.Name, true) != 0 &&
-				MainWindow.Toplevel.Database.Tags.GetTagByName (args.NewText) != null) {
+		    tag_store.GetTagByName (args.NewText) != null) {
 			HigMessageDialog md = new HigMessageDialog (MainWindow.Toplevel.Window,
 				DialogFlags.DestroyWithParent, 
 				MessageType.Warning, ButtonsType.Ok, 
@@ -576,44 +632,50 @@ public class TagSelectionWidget : TreeView {
 		}
 
 		tag.Name = args.NewText;
-		MainWindow.Toplevel.Database.Tags.Commit (tag);
+		tag_store.Commit (tag);
+
+		text_render.Edited -= HandleTagNameEdited;
 
 		args.RetVal = true;
 		return;
 	}
 
-	TreeViewColumn check_column;
-	TreeViewColumn icon_column;
-	TreeViewColumn name_column;
+	//TreeViewColumn check_column;
+	//TreeViewColumn icon_column;
+	//TreeViewColumn name_column;
+	CellRendererPixbuf pix_render;
+	TreeViewColumn complete_column;
+	CellRendererText text_render;
 
 	// Constructor.
 	public TagSelectionWidget (TagStore tag_store)
-		: base (new TreeStore (typeof (uint), typeof(string)))
+		: base (new TreeStore (typeof(uint), typeof(string)))
 	{
 		HeadersVisible = false;
 		Selection.Mode = SelectionMode.Multiple;
 
-		CellRendererToggle toggle_renderer = new CellRendererToggle ();
-		toggle_renderer.Toggled += new ToggledHandler (OnCellToggled);
+		complete_column = new TreeViewColumn ();
+				
+		pix_render = new CellRendererPixbuf ();
+		complete_column.PackStart (pix_render, false);
+		complete_column.SetCellDataFunc (pix_render, new TreeCellDataFunc (IconDataFunc));
+		//complete_column.AddAttribute (pix_render, "pixbuf", OpenIconColumn);
 
-		check_column = AppendColumn ("check", toggle_renderer, new TreeCellDataFunc (CheckBoxDataFunc));
-		check_column.SortColumnId = 0;
+		//icon_column = AppendColumn ("icon", 
+		//, new TreeCellDataFunc (IconDataFunc));
+		//icon_column = AppendColumn ("icon", new CellRendererPixbuf (), new TreeCellDataFunc (IconDataFunc));
 
-		icon_column = AppendColumn ("icon", new CellRendererPixbuf (), new TreeCellDataFunc (IconDataFunc));
+		text_render = new CellRendererText ();
+		complete_column.PackStart (text_render, true);
+		complete_column.SetCellDataFunc (text_render, new TreeCellDataFunc (NameDataFunc));
 
-		CellRendererText tr = new CellRendererText ();
-		tr.Editable = true;
-		tr.Edited += HandleTagNameEdited;
-		tr.Mode = CellRendererMode.Editable;
+		AppendColumn (complete_column);
 
-		name_column = AppendColumn ("name", tr, new TreeCellDataFunc (NameDataFunc));
-		name_column.SortColumnId = 1;
-		
 		this.tag_store = tag_store;
 		selection = new Hashtable ();
 
 		Update ();
-		
+
 		ExpandDefaults ();
 
 		tag_store.ItemsAdded += HandleTagsAdded;
@@ -622,7 +684,7 @@ public class TagSelectionWidget : TreeView {
 
 		// TODO make the search find tags that are not currently expanded
 		EnableSearch = true;
-		SearchColumn = 1;
+		SearchColumn = NameColumn;
 
 		// Transparent white
 		empty_pixbuf.Fill(0xffffff00);
